#
# SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
# Copyright (C) - Telecom ParisTech
# Contacts: contact-secbus@telecom-paristech.fr
#
# This file must be used under the terms of the CeCILL.
# This source file is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at
# http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
#

# This Makefile is to be included by others. It handles the production of SD card
# tarballs for the target board (ZedBoard only for the current version).

# Debugging
DEBUG	?= 1
ifeq ($(DEBUG),0)
OUTPUT	= &> /dev/null
else ifeq ($(DEBUG),1)
OUTPUT	= > /dev/null
else
undefine OUTPUT
endif

help: lhelp

define LHELP_message
--------------------------------------------------------------------------------
"make sdcard-help": print help about production of SD card archives.
endef
export LHELP_message

lhelp:
	@echo "$$LHELP_message"


define SDCARDHELP_message
--------------------------------------------------------------------------------
               HELP ABOUT THE PRODUCTION OF SD CARDS ARCHIVES
--------------------------------------------------------------------------------
Make variables common to all SD card archives (default value):

DEBUG              debug level: 0=none, 1: some, 2: verbose (1)
TOP                VHDL top level design unit (no default, must be defined)
XLNXROOTDIR        root directory of clones of Xilinx git repositories (/opt/xlnx)
XLNXLINUXDIR       clone of Xilinx Linux git repository ($$(XLNXROOTDIR)/linux-xlnx)
XLNXUBOOTDIR       clone of Xilinx U-Boot git repository ($$(XLNXROOTDIR)/u-boot-xlnx)
XLNXDEVICETREEDIR  clone of Xilinx device trees git repository ($$(XLNXROOTDIR)/device-tree-xlnx)
MKIMAGE            U-Boot mkimage utility ($$(XLNXUBOOTDIR)/build/tools/mkimage)
DTC                device tree compiler ($$(XLNXLINUXDIR)/build/scripts/dtc/dtc)
SDCARDS            list of SD card archive names (sdcard-$$(TOP))
ZIMAGE             compressed Linux kernel image ($$(XLNXLINUXDIR)/build/arch/arm/boot/zImage)
UILA               load address to use when creating U-Boot image of kernel (0x8000)
FCLKEN             FPGA clock enables, for device tree generation (0x1)
KNBA               base address of physical memory usable by kernel, for device tree generation (0x0)
KNMS               byte-size of physical memory usable by kernel, for device tree generation (0x20000000)
BA                 bootargs to pass the kernel when booting (console=ttyPS0,115200)
L2CACHE            L2 cache enabled, for device tree generation (yes)
ROOTFS             root filesystem image ($$(XLNXROOTDIR)/arm_ramdisk.image.gz)
EXTRA              extra files or directories to archive (uEnv.txt README .bashrc COPYING COPYING-FR)

To create SD card archives, add the archive name (AAA) to the SDCARDS make
variable. The following variables are then automatically defined:

AAA_TOP            toplevel VHDL design unit ($$(TOP))
AAA_UBOOTELF       U-Boot ELF ($$(XLNXUBOOTDIR)/build/u-boot)
AAA_SYSDEF         hardware system definition ($$(AAA_TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef)
AAA_BITFILE        bitstream ($$(AAA_TOP).vv-syn/top.runs/impl_1/top_wrapper.bit
AAA_UIMAGES        list of Linux kernels (kernel)
AAA_DEVICETREES    list of device trees (devicetree)
AAA_ROOTFSS        list of root filesystems (ramdisk)
AAA_EXTRA          extra files to archive ($$(EXTRA))

For each kernel KKK in $$(AAA_UIMAGES):
AAA_KKK_ZIMAGE     compressed Linux kernel image ($$(ZIMAGE))
AAA_KKK_UILA       load address to use when creating U-Boot image of kernel ($$(UILA))

For each device tree DDD in $$(AAA_DEVICETREES):
AAA_DDD_FCLKEN     FPGA clocks enable ($$(FCLKEN))
AAA_DDD_KNBA       base address of physical memory usable by kernel ($$(KNBA))
AAA_DDD_KNMS       byte-size of physical memory usable by kernel ($$(KNMS))
AAA_DDD_BA         bootargs to pass the kernel when booting ($$(BA))
AAA_DDD_L2CACHE    L2 cache enabled (yes/no) ($$(L2CACHE))

For each root filesystem RRR in $$(AAA_ROOTFSS):
AAA_RRR_ROOTFS     root filesystem image ($$(ROOTFS))

The following make targets are automatically defined:
AAA.tgz                        compressed tarball (to unpack on first FAT32 partition of SD card)
AAA/tarball/boot.bin           boot image (FSBL, bitstream, U-Boot)
AAA/tarball/src/fsbl.elf       copy of FSBL ELF (from SDK)
AAA/tarball/src/top.hdf        copy of hardware description file (from Vivado)
AAA/tarball/src/bitstream.bit  copy of bitstream (from Vivado)
AAA/tarball/src/u-boot.elf     copy of U-Boot ELF
AAA/tarball/KKK.uimage         U-Boot image of kernel KKK
AAA/tarball/DDD.dtb            blob of device tree DDD
AAA/tarball/RRR.rootfs         U-Boot image of root filesystem image RRR
AAA/tarball/EEE                copy of extra EEE

The following make targets are also automatically defined but less useful:
AAA/fsbl/executable.elf        FSBL ELF (from SDK)
AAA/sdk/top.hdf                hardware description file (from Vivado): FSBL ELF

Finally, the following targets can be used to clean up:
sdcard-clean                   clean log and journal files
sdcard-ultraclean              delete SD card archives and tarballs
--------------------------------------------------------------------------------
Example for the production of one single SD card archive 'sda' with all defaults
except:
- VHDL top level design unit: axi_bridge_wrapper
- Xilinx git clones in default locations (/opt/xlnx/{u-boot-xlnx, linux-xlnx, device-tree-xlnx)
- Four Linux kernels with/without CPU caches enabled and in regular address
  space - ras or alternate address space - aas (zImage, UILA):
  + cache-ras ($$(XLNXLINUXDIR)/build-cache/arch/arm/boot/zImage, 0x8000)
  + cache-aas ($$(XLNXLINUXDIR)/build-cache/arch/arm/boot/zImage, 0x48008000)
  + nocache-ras ($$(XLNXLINUXDIR)/build-nocache/arch/arm/boot/zImage, 0x8000)
  + nocache-aas ($$(XLNXLINUXDIR)/build-nocache/arch/arm/boot/zImage, 0x48008000)
- One device tree per kernel (KNBA, KNMS, L2CACHE):
  + cache-ras (0x0, 0x20000000, yes)
  + cache-aas (0x48000000, 0x18000000, yes)
  + nocache-ras (0x0, 0x20000000, no)
  + nocache-aas (0x48000000, 0x18000000, no)
- Two root filesystems (image):
  + nopython (/opt/buildroot/secbus/images/rootfs.cpio.gz)
  + python (/opt/buildroot/secbus-python/images/rootfs.cpio.gz)

make TOP=axi_bridge_wrapper \ 
	SDCARDS=sda \ 
	sda_UIMAGES="cache-ras cache-aas nocache-ras nocache-aas" \ 
	sda_DEVICETREES="cache-ras cache-aas nocache-ras nocache-aas" \ 
	sda_ROOTFSS="nopython python" \ 
	sda_cache-ras_ZIMAGE=/opt/xlnx/linux-xlnx/build-cache/arch/arm/boot/zImage \ 
	sda_cache-ras_ZIMAGE=/opt/xlnx/linux-xlnx/build-nocache/arch/arm/boot/zImage \ 
	sda_cache-aas_ZIMAGE=/opt/xlnx/linux-xlnx/build-cache/arch/arm/boot/zImage \ 
	sda_cache-aas_ZIMAGE=/opt/xlnx/linux-xlnx/build-nocache/arch/arm/boot/zImage \ 
	sda_cache-aas_UILA=0x48008000 \ 
	sda_cache-aas_UILA=0x48008000 \ 
	sda_cache-aas_KNBA=0x48000000 \ 
	sda_cache-aas_KNMS=0x18000000 \ 
	sda_nocache-ras_L2CACHE=no \ 
	sda_nocache-aas_KNBA=0x48000000 \ 
	sda_nocache-aas_KNMS=0x18000000 \ 
	sda_nocache-aas_L2CACHE=no \ 
	sda_nopython_ROOTFS=/opt/buildroot/secbus-mini/images/rootfs.cpio.gz \ 
	sda_python_ROOTFS=/opt/buildroot/secbus/images/rootfs.cpio.gz \ 
	sda.tgz

will produce the expected archive (of course, all these make variable
definitions are better declared in the Makefile that includes sdcard.mk). If the
uEnv.txt README, .bashrc, COPYING or COPYING-FR files exists, they will also be
included at the root of the archive.  A reasonable uEnv.txt file with
definitions to boot with one of eight configurations (cache/nocache, ras/aas,
python/nopython), for instance, could be:

### uEnv.txt ###
fdt_high=0xffffffff
initrd_high=0xffffffff

dtb-cache-ras=dtb-cache-ras.dtb
dtb-cache-ras_la=0x02a00000
kernel-cache-ras=cache-ras.uImage
kernel-cache-ras_la=0x03000000

dtb-cache-aas=dtb-cache-aas.dtb
dtb-cache-aas_la=0x4aa00000
kernel-cache-aas=cache-aas.uImage
kernel-cache-aas_la=0x4b000000

dtb-nocache-ras=dtb-nocache-ras.dtb
dtb-nocache-ras_la=0x02a00000
kernel-nocache-ras=nocache-ras.uImage
kernel-nocache-ras_la=0x03000000

dtb-nocache-aas=dtb-nocache-aas.dtb
dtb-nocache-aas_la=0x4aa00000
kernel-nocache-aas=nocache-aas.uImage
kernel-nocache-aas_la=0x4b000000

rootfs-python=python.rootfs
rootfs-nopython=nopython.rootfs
rootfs-ras_la=0x02000000
rootfs-aas_la=0x4a000000

sdboot-cache-ras-nopython=load mmc 0 $${kernel-cache-ras_la} $${kernel-cache-ras} && load mmc 0 $${dtb-cache-ras_la} $${dtb-cache-ras} && load mmc 0 $${rootfs-ras_la} $${rootfs-nopython} && bootm $${kernel-cache-ras_la} $${rootfs-ras_la} $${dtb-cache-ras_la}
sdboot-cache-ras-python=load mmc 0 $${kernel-cache-ras_la} $${kernel-cache-ras} && load mmc 0 $${dtb-cache-ras_la} $${dtb-cache-ras} && load mmc 0 $${rootfs-ras_la} $${rootfs-python} && bootm $${kernel-cache-ras_la} $${rootfs-ras_la} $${dtb-cache-ras_la}
sdboot-cache-aas-nopython=load mmc 0 $${kernel-cache-aas_la} $${kernel-cache-aas} && load mmc 0 $${dtb-cache-aas_la} $${dtb-cache-aas} && load mmc 0 $${rootfs-aas_la} $${rootfs-nopython} && bootm $${kernel-cache-aas_la} $${rootfs-aas_la} $${dtb-cache-aas_la}
sdboot-cache-aas-python=load mmc 0 $${kernel-cache-aas_la} $${kernel-cache-aas} && load mmc 0 $${dtb-cache-aas_la} $${dtb-cache-aas} && load mmc 0 $${rootfs-aas_la} $${rootfs-python} && bootm $${kernel-cache-aas_la} $${rootfs-aas_la} $${dtb-cache-aas_la}
sdboot-nocache-ras-nopythonmmcinfo; then run uenvboot;load mmc 0 $${kernel-nocache-ras_la} $${kernel-nocache-ras} && load mmc 0 $${dtb-nocache-ras_la} $${dtb-nocache-ras} && load mmc 0 $${rootfs-ras_la} $${rootfs-nopython} && bootm $${kernel-nocache-ras_la} $${rootfs-ras_la} $${dtb-nocache-ras_la}
sdboot-nocache-ras-python=load mmc 0 $${kernel-nocache-ras_la} $${kernel-nocache-ras} && load mmc 0 $${dtb-nocache-ras_la} $${dtb-nocache-ras} && load mmc 0 $${rootfs-ras_la} $${rootfs-python} && bootm $${kernel-nocache-ras_la} $${rootfs-ras_la} $${dtb-nocache-ras_la}
sdboot-nocache-aas-nopython=load mmc 0 $${kernel-nocache-aas_la} $${kernel-nocache-aas} && load mmc 0 $${dtb-nocache-aas_la} $${dtb-nocache-aas} && load mmc 0 $${rootfs-aas_la} $${rootfs-nopython} && bootm $${kernel-nocache-aas_la} $${rootfs-aas_la} $${dtb-nocache-aas_la}
sdboot-nocache-aas-python=load mmc 0 $${kernel-nocache-aas_la} $${kernel-nocache-aas} && load mmc 0 $${dtb-nocache-aas_la} $${dtb-nocache-aas} && load mmc 0 $${rootfs-aas_la} $${rootfs-python} && bootm $${kernel-nocache-aas_la} $${rootfs-aas_la} $${dtb-nocache-aas_la}
### end uEnv.txt ###

endef
export SDCARDHELP_message

sdcard-help:
	@echo "$$SDCARDHELP_message"

# Default locations for the Xilinx Linux kernel, U-Boot and device tree sources
# git clones.
XLNXROOTDIR		?= /opt/xlnx
XLNXLINUXDIR		?= $(XLNXROOTDIR)/linux-xlnx
XLNXUBOOTDIR		?= $(XLNXROOTDIR)/u-boot-xlnx
XLNXDEVICETREEDIR	?= $(XLNXROOTDIR)/device-tree-xlnx

# Tools
HSI		= hsi
HSIFSBLSCRIPT	= $(scriptsdir)/fsbl.hsi.tcl
HSIFSBLPATCH	?= $(scriptsdir)/fsbl.patch
HSIFSBLFLAGS	= -mode batch -quiet -notrace -source $(HSIFSBLSCRIPT)
HSIDTSSCRIPT	= $(scriptsdir)/dts.hsi.tcl
HSIDTSFLAGS	= -mode batch -quiet -notrace -source $(HSIDTSSCRIPT)

BOOTGEN		= bootgen
BIF		?= $(scriptsdir)/boot.bif
BOOTGENFLAGS	= -w -image $(BIF)

TAR		= tar
TARFLAGS	= -zc --dereference

# Default mkimage U-Boot utility
MKIMAGE			?= $(XLNXUBOOTDIR)/build/tools/mkimage
MKIMAGELINUXFLAGS	= -A arm -O linux -C none -T kernel
MKIMAGEROOTFSFLAGS	= -A arm -T ramdisk -C gzip

# Default device tree compiler
DTC		?= $(XLNXLINUXDIR)/build/scripts/dtc/dtc
DTCFLAGS	= -I dts -O dtb

# Default list of target SD card archives
SDCARDS		?= sdcard-$(TOP)
SDCARDSTGZ	= $(patsubst %,%.tgz,$(SDCARDS))

# Default zImage for all Linux kernels
ZIMAGE		?= $(XLNXLINUXDIR)/build/arch/arm/boot/zImage
# Default U-Boot load address and entry point (mkimage -a and -e options) for
# all Linux kernels
UILA		?= 0x8000

# Default parameters for device trees
# FPGA clock enables (4 clocks => 4 bits)
FCLKEN	?= 0x1
# Linux kernel base address of physical memory
KNBA	?= 0x0
# Linux kernel size of physical memory
KNMS	?= 0x20000000
# Boot arguments (defines the bootargs variable of device tree)
BA	?= console=ttyPS0,115200
# With or without L2 cache
L2CACHE	?= yes

# Default root filesystem image for all root filesystems
ROOTFS	?= $(XLNXROOTDIR)/arm_ramdisk.image.gz

# Default extra files
EXTRA	?= $(wildcard uEnv.txt) $(wildcard README) $(wildcard .bashrc) $(wildcard COPYING) $(wildcard COPYING-FR)

# Error messages
define SYSDEF_message
**************************************************************
System definition file $(2) not found.

The system definition file is searched in different places
depending on make variables:
- $$($(1)_SYSDEF) if $(1)_SYSDEF defined
- else $$($(1)_TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef if $(1)_TOP defined
- else $$(TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef if TOP defined

So, please define at least one of these make variables such
that it points to a valid system definition file. If needed,
first synthesize your design:

make <top-level>.vsyn

Then, run one of:

make $(1)_SYSDEF=<top-level>.vv-syn/top.runs/impl_1/top_wrapper.sysdef $(1).tgz
make $(1)_TOP=<top-level> $(1).tgz
make TOP=<top-level> $(1).tgz
**************************************************************
endef

define BITFILE_message
**************************************************************
Bitstream $(2) not found.

The bitstream is searched in different places depending on
make variables:
- $$($(1)_BITFILE) if $(1)_BITFILE defined
- else $$($(1)_TOP).vv-syn/top.runs/impl_1/top_wrapper.bit if $(1)_TOP defined
- else $$(TOP).vv-syn/top.runs/impl_1/top_wrapper.bit if TOP defined

So, please define at least one of these make variables such
that it points to a valid bitstream. If needed, first
synthesize your design:

make <top-level>.vsyn

Then, run one of:

make $(1)_BITFILE=<top-level>.vv-syn/top.runs/impl_1/top_wrapper.bit $(1).tgz
make $(1)_TOP=<top-level> $(1).tgz
make TOP=<top-level> $(1).tgz
**************************************************************
endef

define UBOOT_message
**************************************************************
U-Boot ELF file $(2) not found.

The U-Boot ELF file is searched in different places depending
on make variables:
- $$($(1)_UBOOTEFL) if $(1)_UBOOTEFL defined
- else $$(XLNXUBOOTDIR)/build/u-boot if XLNXUBOOTDIR defined
- else $$(XLNXROOTDIR)/u-boot-xlnx/build/u-boot if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid U-Boot ELF file. If needed, first
clone the Xilinx U-Boot git repository and build U-Boot:

git clone http://github.com/Xilinx/u-boot-xlnx.git <path>
cd <path>
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make O=build zynq_zed_defconfig
make O=build menuconfig
  Workaround the SPL bug:
    Boot images  ---> Enable SPL ---> No
  Workaround the RSA-related bug:
    Boot images  ---> Enable signature verification of FIT uImages ---> No
    Library routines  --->  Use RSA Library ---> No
make O=build

Then, run one of:

make XLNXROOTDIR=<path>/.. $(1).tgz
make XLNXUBOOTDIR=<path> $(1).tgz
make $(1)_UBOOTELF=<path>/build/u-boot $(1).tgz
**************************************************************
endef

define MKIMAGE_message
**************************************************************
Mkimage $(2) U-Boot utility not found.

The mkimage U-Boot utility is searched in different places
depending on make variables:
- $$(MKIMAGE) if MKIMAGE defined
- else $$(XLNXUBOOTDIR)/build/tools/mkimage if XLNXUBOOTDIR defined
- else $$(XLNXROOTDIR)/u-boot-xlnx/tools/mkimage if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid U-Boot mkimage utility. If needed,
first clone the Xilinx U-Boot git repository and build U-Boot:

git clone http://github.com/Xilinx/u-boot-xlnx.git <path>
cd <path>
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make O=build zynq_zed_defconfig
make O=build menuconfig
  Workaround the SPL bug:
    Boot images  ---> Enable SPL ---> No
  Workaround the RSA-related bug:
    Boot images  ---> Enable signature verification of FIT uImages ---> No
    Library routines  --->  Use RSA Library ---> No
make O=build

Then, run one of:

make XLNXROOTDIR=<path>/.. $(1).tgz
make XLNXUBOOTDIR=<path> $(1).tgz
make MKIMAGE=<path>/build/tools/mkimage $(1).tgz
**************************************************************
endef

define ZIMAGE_message
**************************************************************
Compressed Linux kernel image $(3) not found.

The Linux kernel zImage is searched in different places
depending on make variables:
- $$($(1)_$(2)_ZIMAGE) if $(1)_$(2)_ZIMAGE defined
- else $$(XLNXLINUXDIR)/build/arch/arm/boot/zImage if XLNXLINUXDIR defined
- else $$(XLNXROOTDIR)/linux-xlnx/build/arch/arm/boot/zImage if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid Linux kernel zImage. If needed, first
clone the Xilinx Linux kernel git repository and build Linux:

git clone http://github.com/Xilinx/u-boot-xlnx.git <path>
cd <path>
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make ARCH=arm O=build xilinx_zynq_defconfig
make ARCH=arm O=build menuconfig
make ARCH=arm O=build zImage

Then, run one of:

make XLNXROOTDIR=<path>/.. $(1).tgz
make XLNXUBOOTDIR=<path> $(1).tgz
make $(1)_$(2)_ZIMAGE=<path>/build/arch/arm/boot/zImage $(1).tgz
**************************************************************
endef

define DTC_message
**************************************************************
Device tree compiler $(2) not found.

The dtc utility is searched in different places depending on
make variables:
- $$(DTC) if DTC defined
- else $$(XLNXLINUXDIR)/build/scripts/dtc/dtc if XLNXLINUXDIR defined
- else $$(XLNXROOTDIR)/linux-xlnx/build/scripts/dtc/dtc if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid dtc utility. If needed, first clone
the Xilinx Linux git repository and build the Linux kernel:

git clone http://github.com/Xilinx/linux-xlnx.git <path>
cd <path>
export CROSS_COMPILE=arm-xilinx-linux-gnueabi-
make ARCH=arm O=build xilinx_zynq_defconfig
make ARCH=arm O=build menuconfig
make ARCH=arm O=build zImage

Then, run one of:

make XLNXROOTDIR=<path>/.. $(1).tgz
make XLNXLINUXDIR=<path> $(1).tgz
make DTC=<path>/build/scripts/dtc/dtc $(1).tgz
**************************************************************
endef

define DTS_message
**************************************************************
Device tree source directory $(1) already exists.

Cowardly refusing to modify it. To re-create it from scratch,
please delete it first and run make again. Continuing with the
existing device tree sources.
**************************************************************
endef

define XLNXDTS_message
**************************************************************
Xilinx device tree source directory $(2) not found.

The Xilinx device tree source directory is searched in
different places depending on make variables:
- $$(XLNXDEVICETREEDIR) if XLNXDEVICETREEDIR defined
- else $$(XLNXROOTDIR)/device-tree-xlnx if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid Xilinx device trees source directory.

If needed, first clone the Xilinx device trees git repository:

git clone http://github.com/Xilinx/device-tree-xlnx.git <path>

Then, run one of:

make XLNXROOTDIR=<path>/.. $(1).tgz
make XLNXDEVICETREEDIR=<path>/device-tree-xlnx $(1).tgz
**************************************************************
endef

define ROOTFS_message
**************************************************************
Root filesystem image $(3) not found.

The root filesystem image is searched in different places
depending on make variables:
- $$($(1)_$(2)_ROOTFS) if $(1)_$(2)_ROOTFS defined
- else $$(XLNXROOTDIR)/arm_ramdisk.image.gz if XLNXROOTDIR defined

So, please define at least one of these make variables such
that it points to a valid root filesystem image. If needed,
first download the Xilinx ramdisk image:

cd <path>
wget http://www.wiki.xilinx.com/file/view/arm_ramdisk.image.gz

Then, run one of:

make XLNXROOTDIR=<path> $(1).tgz
make $(1)_$(2)_ROOTFS=<path>/arm_ramdisk.image.gz $(1).tgz

Of course, you can also create a root filesystem (ramdisk or
initramfs) by other means like, for instance, buildroot.
**************************************************************
endef

define EXTRA_message
**************************************************************
Extra $(2) already exists in tarball directory $(1)/tarball.

Cowardly refusing to replace it. To force replacing, please
delete it first and run make again. Continuing with the
existing extra.
**************************************************************
endef

# Create a Linux kernel U-Boot image
define UIMAGE_create
$(1)_$(2)_ZIMAGE	?= $(ZIMAGE)
$(1)_$(2)_UILA		?= $(UILA)

.PHONY: $(1)/tarball/$(2).uImage

$(1)/tarball/$(2).uImage: my_MKIMAGE_message = $$(call MKIMAGE_message,$(1),$$(MKIMAGE))
$(1)/tarball/$(2).uImage: my_ZIMAGE_message = $$(call ZIMAGE_message,$(1),$(2),$$($(1)_$(2)_ZIMAGE))

$(1)/tarball/$(2).uImage:
	@if [ ! -x $$(MKIMAGE) ]; then \
		export my_MKIMAGE_message; \
		echo "$$$$my_MKIMAGE_message"; \
		exit -1; \
	fi; \
	if [ ! -f $$($(1)_$(2)_ZIMAGE) ]; then \
		export my_ZIMAGE_message; \
		echo "$$$$my_ZIMAGE_message"; \
		exit -1; \
	fi; \
	mkdir -p $$(dir $$@); \
	echo '[MKIMAGE] $$($(1)_$(2)_ZIMAGE) --> $$@'; \
	$(MKIMAGE) $(MKIMAGELINUXFLAGS) -a $$($(1)_$(2)_UILA) -e $$($(1)_$(2)_UILA) -d $$($(1)_$(2)_ZIMAGE) $$@ $(OUTPUT)
endef

# Create device tree sources, customize them and compile a device tree blob
define DEVICETREE_create
$(1)_$(2)_FCLKEN	?= $(FCLKEN)
$(1)_$(2)_KNBA		?= $(KNBA)
$(1)_$(2)_KNMS		?= $(KNMS)
$(1)_$(2)_BA		?= $(BA)
$(1)_$(2)_L2CACHE	?= $(L2CACHE)
$(1)_$(2)_DTSDIR	= $(1)/tarball/src/$(2)_dts
$(1)_$(2)_DTS		= $(1)/tarball/src/$(2)_dts/system.dts

.PHONY: $(1)/tarball/src/$(2)_dts/system.dts

$$($(1)_$(2)_DTS): my_DTS_message = $$(call DTS_message,$(1)/tarball/src/$(2)_dts)
$$($(1)_$(2)_DTS): my_XLNXDTS_message = $$(call XLNXDTS_message,$(1),$$(XLNXDEVICETREEDIR))

$$($(1)_$(2)_DTS): $(1)/sdk/top.hdf $(HSIDTSSCRIPT)
	@if [ -d $$($(1)_$(2)_DTSDIR) ]; then \
		export my_DTS_message; \
		echo "$$$$my_DTS_message"; \
	else \
		if [ ! -d $$(XLNXDEVICETREEDIR) ]; then \
			export my_XLNXDTS_message; \
			echo "$$$$my_XLNXDTS_message"; \
			exit -1; \
		fi; \
		mkdir -p $$($(1)_$(2)_DTSDIR); \
		echo '[HSI] $(1)/sdk/top.hdf --> $$($(1)_$(2)_DTSDIR)'; \
		$(HSI) $(HSIDTSFLAGS) -tclargs $(1)/sdk/top.hdf $(XLNXDEVICETREEDIR) $$($(1)_$(2)_DTSDIR) $(OUTPUT); \
		echo '[SED] $$($(1)_$(2)_DTS) --> $$($(1)_$(2)_DTS)'; \
		sed --in-place --regexp-extended \
		-e '/^\s*memory\s*\{$$$$/ {:a;N;/^(\s*)memory\s*\{(\n.*)*\n\1\};$$$$/s#\n(\s*)reg\s*=[^\n]*#\n\1linux,usable-memory = <$$($(1)_$(2)_KNBA) $$($(1)_$(2)_KNMS)>;#;Ta;};' $$($(1)_$(2)_DTS); \
		sed --in-place --regexp-extended \
		-e '/^\s*chosen\s*\{$$$$/ {:a;N;/^(\s*)chosen\s*\{(\n.*)*\n\1\};$$$$/s#\n(\s*)bootargs\s*=[^\n]*#\n\1bootargs = "$$($(1)_$(2)_BA)";#;Ta;};' $$($(1)_$(2)_DTS); \
		sed --in-place --regexp-extended \
		-e '/^\s*&clkc\s*\{$$$$/ {:a;N;/^(\s*)&clkc\s*\{(\n.*)*\n\1\};$$$$/s#\n(\s*)fclk-enable\s*=[^\n]*#\n\1fclk-enable = <$$($(1)_$(2)_FCLKEN)>;#;Ta;};' $$($(1)_$(2)_DTS); \
		if [ "X$$($(1)_$(2)_L2CACHE)" != "Xyes" ]; then \
			echo '[SED] $$($(1)_$(2)_DTSDIR)/zynq-7000.dtsi --> $$($(1)_$(2)_DTSDIR)/zynq-7000.dtsi'; \
			sed --in-place --regexp-extended \
			-e '/^\s*L2:\s*cache-controller\@f8f02000\s*\{$$$$/ {:a;N;/^(\s*)L2:\s*cache-controller\@f8f02000\s*\{(\n.*)*\n\1\};$$$$/d;Ta;};' $$($(1)_$(2)_DTSDIR)/zynq-7000.dtsi; \
		fi; \
	fi

$(1)/tarball/$(2).dtb: my_DTC_message = $$(call DTC_message,$(1),$$(DTC))

$(1)/tarball/$(2).dtb: $$($(1)_$(2)_DTS)
	@if [ ! -x $$(DTC) ]; then \
		export my_DTC_message; \
		echo "$$$$my_DTC_message"; \
		exit -1; \
	fi; \
	echo '[DTC] $$< --> $$@'; \
	$(DTC) $(DTCFLAGS) -o $(1)/tarball/$(2).dtb $$($(1)_$(2)_DTS) $(OUTPUT)
endef

# Create a rootfs U-Boot image
define ROOTFS_create
$(1)_$(2)_ROOTFS	?= $(ROOTFS)

.PHONY: $(1)/tarball/$(2).rootfs

$(1)/tarball/$(2).rootfs: my_MKIMAGE_message = $$(call MKIMAGE_message,$(1),$$(MKIMAGE))
$(1)/tarball/$(2).rootfs: my_ROOTFS_message = $$(call ROOTFS_message,$(1),$(2),$$($(1)_$(2)_ROOTFS))

$(1)/tarball/$(2).rootfs:
	@if [ ! -x $$(MKIMAGE) ]; then \
		export my_MKIMAGE_message; \
		echo "$$$$my_MKIMAGE_message"; \
		exit -1; \
	fi; \
	if [ ! -f $$($(1)_$(2)_ROOTFS) ]; then \
		export my_ROOTFS_message; \
		echo "$$$$my_ROOTFS_message"; \
		exit -1; \
	fi; \
	echo '[MKIMAGE] $$($(1)_$(2)_ROOTFS) --> $$@'; \
	$(MKIMAGE) $(MKIMAGEROOTFSFLAGS) -d $$($(1)_$(2)_ROOTFS) $$@ $(OUTPUT)
endef

# Copy extra to tarball sub-directory
define EXTRA_create
$(1)/tarball/$(2): my_EXTRA_message = $$(call EXTRA_message,$(1),$(2))
$(1)/tarball/$(2): $(3)
	@if [ -e $$@ ]; then \
		export my_EXTRA_message; \
		echo "$$$$my_EXTRA_message"; \
	else \
		mkdir -p $$(dir $$@); \
		echo '[CP] $$< --> $$@'; \
		cp $$< $$@; \
	fi
endef

# Wrap up and create SD card archive
define SDCARD_archive
$(1)_TOP		?= $(TOP)
$(1)_UBOOTELF		?= $(XLNXUBOOTDIR)/build/u-boot
$(1)_SYSDEF		?= $$($(1)_TOP).vv-syn/top.runs/impl_1/top_wrapper.sysdef
$(1)_BITFILE		?= $$($(1)_TOP).vv-syn/top.runs/impl_1/top_wrapper.bit
$(1)_UIMAGES		?= kernel
$(1)_DEVICETREES	?= devicetree
$(1)_ROOTFSS		?= ramdisk
$(1)_EXTRA		?= $$(EXTRA)

$(1)_TARBALL		= \
	$$(patsubst %,$(1)/tarball/%.uImage,$$($(1)_UIMAGES)) \
	$$(patsubst %,$(1)/tarball/%.dtb,$$($(1)_DEVICETREES)) \
	$$(patsubst %,$(1)/tarball/%.rootfs,$$($(1)_ROOTFSS)) \
	$$(patsubst %,$(1)/tarball/%,$$(notdir $$($(1)_EXTRA))) \
	$(1)/tarball/boot.bin \
	$(1)/tarball/src/top.hdf \
	$(1)/tarball/src/fsbl.elf \
	$(1)/tarball/src/u-boot.elf \
	$(1)/tarball/src/bitstream.bit \
	$(1)/tarball/src/top.hdf

$(1).tgz: $$($(1)_TARBALL)
	@echo '[TAR] $$^ --> $$@'; \
	$(TAR) $(TARFLAGS) -f $(1).tgz --directory $(1)/tarball .

$(1)/tarball/boot.bin: $(1)/tarball/src/fsbl.elf $(1)/tarball/src/bitstream.bit $(1)/tarball/src/u-boot.elf
	@cd $$(dir $$@)src; \
	echo '[BOOTGEN] $$^ --> $$@'; \
	$(BOOTGEN) $(BOOTGENFLAGS) -o ../boot.bin $(OUTPUT)

.PHONY: $(1)/fsbl/executable.elf

$(1)/fsbl/executable.elf: $(1)/fsbl/.patched
	@echo '[MAKE] $(1)/fsbl'; \
	$(MAKE) -C $(1)/fsbl

$(1)/fsbl/.patched: $(1)/fsbl/main.c $(HSIFSBLPATCH)
	echo '[PATCH] $$(HSIFSBLPATCH) --> $$(1)/fsbl'; \
	patch -p1 -d $(1)/fsbl < $(HSIFSBLPATCH); \
	touch $$@

$(1)/fsbl/main.c: $(1)/sdk/top.hdf $(HSIFSBLSCRIPT)
	@echo '[HSI] $$< --> $$@'; \
	$(HSI) $(HSIFSBLFLAGS) -tclargs $(1)/sdk/top.hdf $(1)/fsbl $(OUTPUT)

$(1)/tarball/src/fsbl.elf: $(1)/fsbl/executable.elf
	@mkdir -p $$(dir $$@); \
	echo '[CP] $$< --> $$@'; \
	cp $$< $$@

$(1)/tarball/src/top.hdf: $(1)/sdk/top.hdf
	@mkdir -p $$(dir $$@); \
	echo '[CP] $$< --> $$@'; \
	cp $$< $$@

$(1)/sdk/top.hdf: my_SYSDEF_message = $$(call SYSDEF_message,$(1),$$($(1)_SYSDEF))

$$($(1)_SYSDEF):
	@export my_SYSDEF_message; \
	echo "$$$$my_SYSDEF_message"; \
	exit -1

$(1)/sdk/top.hdf: $$($(1)_SYSDEF)
	@mkdir -p $$(dir $$@); \
	echo '[CP] $$($(1)_SYSDEF) --> $$@'; \
	cp $$($(1)_SYSDEF) $$@

$(1)/tarball/src/bitstream.bit: my_BITFILE_message = $$(call BITFILE_message,$(1),$$($(1)_BITFILE))

$$($(1)_BITFILE):
	@export my_BITFILE_message; \
	echo "$$$$my_BITFILE_message"; \
	exit -1

$(1)/tarball/src/bitstream.bit: $$($(1)_BITFILE)
	@mkdir -p $$(dir $$@); \
	echo '[CP] $$($(1)_BITFILE) --> $$@'; \
	cp $$($(1)_BITFILE) $$@

$(1)/tarball/src/u-boot.elf: my_UBOOT_message = $$(call UBOOT_message,$(1),$$($(1)_UBOOTELF))

$$($(1)_UBOOTELF):
	@export my_UBOOT_message; \
	echo "$$$$my_UBOOT_message"; \
	exit -1

$(1)/tarball/src/u-boot.elf: $$($(1)_UBOOTELF)
	@mkdir -p $$(dir $$@); \
	echo '[CP] $$($(1)_UBOOTELF) --> $$@'; \
	cp $$($(1)_UBOOTELF) $$@

$$(foreach uimage,$$($(1)_UIMAGES),$$(eval $$(call UIMAGE_create,$(1),$$(uimage))))

$$(foreach devicetree,$$($(1)_DEVICETREES),$$(eval $$(call DEVICETREE_create,$(1),$$(devicetree))))

$$(foreach rootfs,$$($(1)_ROOTFSS),$$(eval $$(call ROOTFS_create,$(1),$$(rootfs))))

$$(foreach extra,$$($(1)_EXTRA),$$(eval $$(call EXTRA_create,$(1),$$(notdir $$(extra)),$$(extra))))
endef

$(foreach sdcard,$(SDCARDS),$(eval $(call SDCARD_archive,$(sdcard))))

clean: sdcard-clean
ultraclean: sdcard-ultraclean

sdcard-clean:
	rm -rf hsi.jou hsi.log hsi_*.backup.jou hsi_*.backup.log

sdcard-ultraclean: sdcard-clean
	rm -rf $(SDCARDS) $(SDCARDSTGZ)
