--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

-- @id $Id: axi432_pkg.vhd 
-- @brief axi432_pkg package.
-- @author Sebastien Cerdan (sebastien.cerdan@telecom-paristech.fr)
-- @date 2013-06-06

library ieee;
use ieee.std_logic_1164.all;

library global_lib;
use global_lib.numeric_std.all;

package axi432_pkg is

  -----------------------------------------------------------------------------------------------------
  -- AXI432 types (AXI4, 32 bits data, 32 bits addresses)
  -----------------------------------------------------------------------------------------------------

  constant axi432_l:      positive := 4;    -- bit width of len
  constant axi432_i:      positive := 6;    -- bit width of id
  constant axi432_d:      positive := 32;   -- bit width of data
  constant axi432_s:      positive := 3;    -- bit width of size
  constant axi432_b:      positive := 2;    -- bit width of burst
  constant axi432_a:      positive := 32;   -- bit width of address
  constant axi432_o:      positive := 2;    -- bit width of lock
  constant axi432_p:      positive := 3;    -- bit width of prot
  constant axi432_c:      positive := 4;    -- bit width of cache
  constant axi432_r:      positive := 2;    -- bit width of resp
  constant axi432_m:      positive := 4;    -- bit width of strb

  -- Address channels
  type axi432_m2s_address_channel_type is record
    aid    : std_ulogic_vector(axi432_i - 1 downto 0);
    aaddr  : std_ulogic_vector(axi432_a - 1 downto 0);      
    alen   : std_ulogic_vector(axi432_l - 1 downto 0);
    asize  : std_ulogic_vector(axi432_s - 1 downto 0);
    aburst : std_ulogic_vector(axi432_b - 1 downto 0);
    alock  : std_ulogic_vector(axi432_o - 1 downto 0);
    acache : std_ulogic_vector(axi432_c - 1 downto 0);
    aprot  : std_ulogic_vector(axi432_p - 1 downto 0);
    avalid : std_ulogic;      
  end record;

  constant axi432_m2s_address_channel_none : axi432_m2s_address_channel_type := 
    (aid    => (others => '0'), 
     aaddr  => (others => '0'), 
     alen   => (others => '0'), 
     asize  => (others => '0'), 
     aburst => (others => '0'), 
     alock  => (others => '0'), 
     acache => (others => '0'), 
     aprot  => (others => '0'), 
     avalid => '0');
 
  type axi432_m2s_read_address_channel_type is record
    arid    : std_ulogic_vector(axi432_i - 1 downto 0);
    araddr  : std_ulogic_vector(axi432_a - 1 downto 0);      
    arlen   : std_ulogic_vector(axi432_l - 1 downto 0);
    arsize  : std_ulogic_vector(axi432_s - 1 downto 0);
    arburst : std_ulogic_vector(axi432_b - 1 downto 0);
    arlock  : std_ulogic_vector(axi432_o - 1 downto 0);
    arcache : std_ulogic_vector(axi432_c - 1 downto 0);
    arprot  : std_ulogic_vector(axi432_p - 1 downto 0);
    arvalid : std_ulogic;      
  end record;
 
  constant axi432_m2s_read_address_channel_none : axi432_m2s_read_address_channel_type := 
    (arid    => (others => '0'), 
     araddr  => (others => '0'), 
     arlen   => (others => '0'), 
     arsize  => (others => '0'), 
     arburst => (others => '0'), 
     arlock  => (others => '0'), 
     arcache => (others => '0'), 
     arprot  => (others => '0'), 
     arvalid => '0'); 

  type axi432_s2m_read_address_channel_type is record
    arready : std_ulogic; 
  end record;     
  
  constant axi432_s2m_read_address_channel_none : axi432_s2m_read_address_channel_type := 
    (arready => '0');

  type axi432_m2s_write_address_channel_type is record
    awid    : std_ulogic_vector(axi432_i - 1 downto 0);
    awaddr  : std_ulogic_vector(axi432_a - 1 downto 0);      
    awlen   : std_ulogic_vector(axi432_l - 1 downto 0);
    awsize  : std_ulogic_vector(axi432_s - 1 downto 0);
    awburst : std_ulogic_vector(axi432_b - 1 downto 0);
    awlock  : std_ulogic_vector(axi432_o - 1 downto 0);
    awcache : std_ulogic_vector(axi432_c - 1 downto 0);
    awprot  : std_ulogic_vector(axi432_p - 1 downto 0);
    awvalid : std_ulogic;      
  end record;

  constant axi432_m2s_write_address_channel_none : axi432_m2s_write_address_channel_type := 
    (awid    => (others => '0'), 
     awaddr  => (others => '0'), 
     awlen   => (others => '0'), 
     awsize  => (others => '0'), 
     awburst => (others => '0'), 
     awlock  => (others => '0'), 
     awcache => (others => '0'), 
     awprot  => (others => '0'), 
     awvalid => '0'); 

  type axi432_s2m_write_address_channel_type is record
    awready : std_ulogic;
  end record;

  constant axi432_s2m_write_address_channel_none : axi432_s2m_write_address_channel_type := 
    (awready => '0');

  function axi432_a2a(c: axi432_m2s_read_address_channel_type) return axi432_m2s_address_channel_type;
  function axi432_a2a(c: axi432_m2s_write_address_channel_type) return axi432_m2s_address_channel_type;

  -- Data channels

  type axi432_s2m_read_data_channel_type is record
    rid      : std_ulogic_vector(axi432_i - 1 downto 0);
    rdata    : std_ulogic_vector(axi432_d - 1 downto 0);
    rresp    : std_ulogic_vector(axi432_r - 1 downto 0);
    rlast    : std_ulogic;
    rvalid   : std_ulogic;      
  end record;

  constant axi432_s2m_read_data_channel_none : axi432_s2m_read_data_channel_type := 
    (rid    => (others => '0'), 
     rdata  => (others => '0'), 
     rresp  => (others => '0'), 
     rlast  => '0', 
     rvalid => '0');
 
  type axi432_m2s_read_data_channel_type is record
    rready   : std_ulogic;
  end record;

  constant axi432_m2s_read_data_channel_none : axi432_m2s_read_data_channel_type := 
    (rready => '0');

  type axi432_m2s_write_data_channel_type is record
    wdata    : std_ulogic_vector(axi432_d - 1 downto 0);
    wstrb    : std_ulogic_vector(axi432_m - 1 downto 0);
    wlast    : std_ulogic;
    wvalid   : std_ulogic;      
  end record;
  
  constant axi432_m2s_write_data_channel_none : axi432_m2s_write_data_channel_type := 
    (wdata  => (others => '0'), 
     wstrb  => (others => '0'), 
     wlast  => '0', 
     wvalid => '0');

  type axi432_s2m_write_data_channel_type is record
    wready   : std_ulogic;
  end record;

  constant axi432_s2m_write_data_channel_none : axi432_s2m_write_data_channel_type := 
    (wready => '0');

  -- Response channels

  type axi432_s2m_write_response_channel_type is record
    bid      : std_ulogic_vector(axi432_i - 1 downto 0);
    bresp    : std_ulogic_vector(axi432_r - 1 downto 0);
    bvalid   : std_ulogic;      
  end record;

  constant axi432_s2m_write_response_channel_none : axi432_s2m_write_response_channel_type :=
    (bid    => (others => '0'), 
     bresp  => (others => '0'), 
     bvalid => '0');
 
  type axi432_m2s_write_response_channel_type is record
    bready   : std_ulogic;
  end record;

  constant axi432_m2s_write_response_channel_none : axi432_m2s_write_response_channel_type :=
    (bready => '0');

  -- End response channels

  type axi432_m2s_type is record                 --Master to slave
    ra : axi432_m2s_read_address_channel_type;   --Read address channel
    wa : axi432_m2s_write_address_channel_type;  --Write address channel
    rd : axi432_m2s_read_data_channel_type;      --Read data channel
    wd : axi432_m2s_write_data_channel_type;     --Write data channel
    wr : axi432_m2s_write_response_channel_type; --Write response channel
  end record;
 
  constant axi432_m2s_none : axi432_m2s_type := 
    (ra => axi432_m2s_read_address_channel_none,
     wa => axi432_m2s_write_address_channel_none,
     rd => axi432_m2s_read_data_channel_none,
     wd => axi432_m2s_write_data_channel_none,
     wr => axi432_m2s_write_response_channel_none);
 
  type axi432_s2m_type is record                 --Slave to master 
    ra : axi432_s2m_read_address_channel_type;   --Read address channel
    wa : axi432_s2m_write_address_channel_type;  --Write address channel
    rd : axi432_s2m_read_data_channel_type;      --Read data channel
    wd : axi432_s2m_write_data_channel_type;     --Write data channel
    wr : axi432_s2m_write_response_channel_type; --Write response channel
  end record;
    
  constant axi432_s2m_none : axi432_s2m_type := 
    (ra => axi432_s2m_read_address_channel_none,
     wa => axi432_s2m_write_address_channel_none,
     rd => axi432_s2m_read_data_channel_none,
     wd => axi432_s2m_write_data_channel_none,
     wr => axi432_s2m_write_response_channel_none);

  constant address_length        : natural := axi432_i + axi432_a + axi432_l + axi432_s + axi432_b + axi432_o + axi432_c + axi432_p + 1;
  constant read_address_length   : natural := axi432_i + axi432_a + axi432_l + axi432_s + axi432_b + axi432_o + axi432_c + axi432_p + 1;
  constant write_address_length  : natural := axi432_i + axi432_a + axi432_l + axi432_s + axi432_b + axi432_o + axi432_c + axi432_p + 1;
  constant read_data_length      : natural := axi432_i + axi432_d + axi432_r + 1 + 1;
  constant write_data_length     : natural := axi432_d + axi432_m + 1 + 1;
  constant write_response_length : natural := axi432_i + axi432_r + 1;
  
  -- Channel to vector functions

  function address_to_vector(a: axi432_m2s_address_channel_type) return std_ulogic_vector;
  function address_to_vector(a: axi432_m2s_read_address_channel_type) return std_ulogic_vector;
  function address_to_vector(a: axi432_m2s_write_address_channel_type) return std_ulogic_vector;

  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_address_channel_type;
  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_read_address_channel_type;
  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_write_address_channel_type;

  function read_address_to_vector(ra: axi432_m2s_read_address_channel_type) return std_ulogic_vector;

  function vector_to_read_address(v : std_ulogic_vector) return axi432_m2s_read_address_channel_type;

  function write_address_to_vector(wa: axi432_m2s_write_address_channel_type) return std_ulogic_vector;

  function vector_to_write_address (v : std_ulogic_vector) return axi432_m2s_write_address_channel_type; 

  function read_data_to_vector(rd: axi432_s2m_read_data_channel_type) return std_ulogic_vector;

  function vector_to_read_data(v : std_ulogic_vector) return axi432_s2m_read_data_channel_type;   

  function write_data_to_vector(wd: axi432_m2s_write_data_channel_type) return std_ulogic_vector;

  function vector_to_write_data(v : std_ulogic_vector) return axi432_m2s_write_data_channel_type;

  function write_response_to_vector(wr: axi432_s2m_write_response_channel_type) return std_ulogic_vector;

  function vector_to_write_response(v : std_ulogic_vector) return axi432_s2m_write_response_channel_type; 

  -- Address management functions

  function get_next_addr(prev_addr, size : std_ulogic_vector) return std_ulogic_vector;
  
  function get_border_addr(wrap_addr, size, len : std_ulogic_vector) return std_ulogic_vector;

  function get_wrap_addr(start_addr, size, len : std_ulogic_vector) return std_ulogic_vector;

  function get_align_addr(start_addr, size : std_ulogic_vector) return std_ulogic_vector;
-- pragma translate_off
  function check_wrap_parameters(addr, size, len : std_ulogic_vector) return std_ulogic;
-- pragma translate_on
  
  function axi432_m2s_address_channel_type2axi432_m2s_read_address_channel_type(v: axi432_m2s_address_channel_type) return axi432_m2s_read_address_channel_type;
  function axi432_m2s_address_channel_type2axi432_m2s_write_address_channel_type(v: axi432_m2s_address_channel_type) return axi432_m2s_write_address_channel_type;
  function axi432_m2s_read_address_channel_type2axi432_m2s_address_channel_type(v: axi432_m2s_read_address_channel_type) return axi432_m2s_address_channel_type;
  function axi432_m2s_write_address_channel_type2axi432_m2s_address_channel_type(v: axi432_m2s_write_address_channel_type) return axi432_m2s_address_channel_type;

end package axi432_pkg;

package body axi432_pkg is

  function axi432_a2a(c: axi432_m2s_read_address_channel_type) return axi432_m2s_address_channel_type is
    variable tmp: axi432_m2s_address_channel_type;
  begin
    tmp.aid    := c.arid;
    tmp.aaddr  := c.araddr;
    tmp.alen   := c.arlen;
    tmp.asize  := c.arsize;
    tmp.aburst := c.arburst;
    tmp.alock  := c.arlock;
    tmp.acache := c.arcache;
    tmp.aprot  := c.arprot;
    tmp.avalid := c.arvalid;
    return tmp;
  end function axi432_a2a;

  function axi432_a2a(c: axi432_m2s_write_address_channel_type) return axi432_m2s_address_channel_type is
    variable tmp: axi432_m2s_address_channel_type;
  begin
    tmp.aid    := c.awid;
    tmp.aaddr  := c.awaddr;
    tmp.alen   := c.awlen;
    tmp.asize  := c.awsize;
    tmp.aburst := c.awburst;
    tmp.alock  := c.awlock;
    tmp.acache := c.awcache;
    tmp.aprot  := c.awprot;
    tmp.avalid := c.awvalid;
    return tmp;
  end function axi432_a2a;

  function address_to_vector(a: axi432_m2s_address_channel_type) return std_ulogic_vector is
    begin
      return a.aid & a.aaddr & a.alen & a.asize & a.aburst & a.alock & a.acache & a.aprot & a.avalid;
  end address_to_vector;

  function address_to_vector(a: axi432_m2s_read_address_channel_type) return std_ulogic_vector is
  begin
    return read_address_to_vector(a);
  end function address_to_vector;

  function address_to_vector(a: axi432_m2s_write_address_channel_type) return std_ulogic_vector is
  begin
    return write_address_to_vector(a);
  end function address_to_vector;

  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_address_channel_type is
  
    variable a   : axi432_m2s_address_channel_type;
    variable tmp : natural;

    begin

      a.avalid := v(0);
      tmp := 1;
      a.aprot  := v(tmp + axi432_p - 1 downto tmp);
      tmp := tmp + axi432_p;
      a.acache := v(tmp + axi432_c - 1 downto tmp);
      tmp := tmp + axi432_c;
      a.alock  := v(tmp + axi432_o - 1 downto tmp);
      tmp := tmp + axi432_o;
      a.aburst := v(tmp + axi432_b - 1 downto tmp);
      tmp := tmp + axi432_b;
      a.asize := v(tmp + axi432_s - 1 downto tmp);
      tmp := tmp + axi432_s;
      a.alen := v(tmp + axi432_l - 1 downto tmp);
      tmp := tmp + axi432_l;
      a.aaddr := v(tmp + axi432_a - 1 downto tmp);
      tmp := tmp + axi432_a;
      a.aid := v(tmp + axi432_i - 1 downto tmp);

      return a;

  end vector_to_address;
 
  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_read_address_channel_type is
  begin
    return vector_to_read_address(v);
  end function vector_to_address;

  function vector_to_address(v : std_ulogic_vector) return axi432_m2s_write_address_channel_type is
  begin
    return vector_to_write_address(v);
  end function vector_to_address;

  function read_address_to_vector(ra: axi432_m2s_read_address_channel_type) return std_ulogic_vector is
    begin
      return ra.arid & ra.araddr & ra.arlen & ra.arsize & ra.arburst & ra.arlock & ra.arcache & ra.arprot & ra.arvalid;
  end read_address_to_vector;

  function vector_to_read_address(v : std_ulogic_vector) return axi432_m2s_read_address_channel_type is
  
    variable ra : axi432_m2s_read_address_channel_type;
    variable tmp : natural;

    begin

      ra.arvalid := v(0);
      tmp := 1;
      ra.arprot  := v(tmp + axi432_p - 1 downto tmp);
      tmp := tmp + axi432_p;
      ra.arcache := v(tmp + axi432_c - 1 downto tmp);
      tmp := tmp + axi432_c;
      ra.arlock  := v(tmp + axi432_o - 1 downto tmp);
      tmp := tmp + axi432_o;
      ra.arburst := v(tmp + axi432_b - 1 downto tmp);
      tmp := tmp + axi432_b;
      ra.arsize := v(tmp + axi432_s - 1 downto tmp);
      tmp := tmp + axi432_s;
      ra.arlen := v(tmp + axi432_l - 1 downto tmp);
      tmp := tmp + axi432_l;
      ra.araddr := v(tmp + axi432_a - 1 downto tmp);
      tmp := tmp + axi432_a;
      ra.arid := v(tmp + axi432_i - 1 downto tmp);

      return ra;

  end vector_to_read_address;

  function write_address_to_vector(wa: axi432_m2s_write_address_channel_type) return std_ulogic_vector is 
    begin
      return  wa.awid & wa.awaddr & wa.awlen & wa.awsize & wa.awburst & wa.awlock & wa.awcache & wa.awprot & wa.awvalid;
  end write_address_to_vector;

  function vector_to_write_address (v : std_ulogic_vector) return axi432_m2s_write_address_channel_type is

    variable wa : axi432_m2s_write_address_channel_type;
    variable tmp : natural;

    begin

      wa.awvalid := v(0);
      tmp := 1;
      wa.awprot  := v(tmp + axi432_p - 1 downto tmp);
      tmp := tmp + axi432_p;
      wa.awcache := v(tmp + axi432_c - 1 downto tmp);
      tmp := tmp + axi432_c;
      wa.awlock  := v(tmp + axi432_o - 1 downto tmp);
      tmp := tmp + axi432_o;
      wa.awburst := v(tmp + axi432_b - 1 downto tmp);
      tmp := tmp + axi432_b;
      wa.awsize := v(tmp + axi432_s - 1 downto tmp);
      tmp := tmp + axi432_s;
      wa.awlen := v(tmp + axi432_l - 1 downto tmp);
      tmp := tmp + axi432_l;
      wa.awaddr := v(tmp + axi432_a - 1 downto tmp);
      tmp := tmp + axi432_a;
      wa.awid := v(tmp + axi432_i - 1 downto tmp);

      return wa;

  end vector_to_write_address;

  function read_data_to_vector(rd: axi432_s2m_read_data_channel_type) return std_ulogic_vector is
    begin
      return rd.rid & rd.rdata & rd.rresp & rd.rlast & rd.rvalid;
  end read_data_to_vector;

  function vector_to_read_data(v : std_ulogic_vector) return axi432_s2m_read_data_channel_type is 

    variable rd: axi432_s2m_read_data_channel_type;
    variable tmp : natural;

    begin

      rd.rvalid := v(0);
      tmp := 1;
      rd.rlast := v(1);
      tmp := 2;
      rd.rresp := v(tmp + axi432_r - 1 downto tmp);
      tmp := tmp + axi432_r;
      rd.rdata := v(tmp + axi432_d - 1 downto tmp);
      tmp := tmp + axi432_d;
      rd.rid := v(tmp + axi432_i - 1 downto tmp);

      return rd;

  end vector_to_read_data;

  function write_data_to_vector(wd: axi432_m2s_write_data_channel_type) return std_ulogic_vector is
    begin
      return wd.wdata & wd.wstrb & wd.wlast & wd.wvalid;
  end write_data_to_vector;

  function vector_to_write_data(v : std_ulogic_vector) return axi432_m2s_write_data_channel_type is 

    variable wd: axi432_m2s_write_data_channel_type;
    variable tmp : natural;

    begin

      wd.wvalid := v(0);
      tmp := 1;
      wd.wlast := v(1);
      tmp := 2;
      wd.wstrb := v(tmp + axi432_m - 1 downto tmp);
      tmp := tmp + axi432_m;
      wd.wdata := v(tmp + axi432_d - 1 downto tmp);

      return wd;

  end vector_to_write_data;

  function write_response_to_vector(wr: axi432_s2m_write_response_channel_type) return std_ulogic_vector is
    begin
      return wr.bid & wr.bresp & wr.bvalid;
  end write_response_to_vector;

  function vector_to_write_response(v : std_ulogic_vector) return axi432_s2m_write_response_channel_type is

    variable wr: axi432_s2m_write_response_channel_type;
    variable tmp : natural;

    begin

      wr.bvalid := v(0);
      tmp := 1;
      wr.bresp := v(tmp + axi432_r - 1 downto tmp);
      tmp := tmp + axi432_r;
      wr.bid := v(tmp + axi432_i - 1 downto tmp);

      return wr;

  end vector_to_write_response;


  function get_align_addr(start_addr, size : std_ulogic_vector) return std_ulogic_vector is

    variable addr : std_ulogic_vector(31 downto 0) := start_addr;

  begin
   
   addr(to_integer(unsigned(size)) - 1 downto 0) := (others => '0');

   return addr;

  end get_align_addr; 
  
  function get_border_addr(wrap_addr, size, len : std_ulogic_vector) return std_ulogic_vector is

  begin
   
   return std_ulogic_vector(unsigned(wrap_addr) + shift_left(resize(unsigned(len), 32), to_integer(unsigned(size))));

  end get_border_addr; 

  function get_next_addr(prev_addr, size : std_ulogic_vector) return std_ulogic_vector is

    variable tmp : unsigned(31 downto 0) := X"00000001";
  begin
   
   return std_ulogic_vector(unsigned(prev_addr) + shift_left(tmp, to_integer(unsigned(size))));

  end get_next_addr; 
     
  function get_wrap_addr(start_addr, size, len : std_ulogic_vector) return std_ulogic_vector is
  
    variable a    : natural;
    variable b    : natural;
    variable addr : std_ulogic_vector(31 downto 0);
 
  begin
   
   addr := start_addr;
   
   case to_integer(unsigned(len)) is 
     when 3 =>
       a := 2;
     when 7 =>
       a := 3;
     when 15 =>
       a := 4;
     when others =>
       a := 1;
   end case;

   b := a + to_integer(unsigned(size));
   
   addr(b - 1 downto 0) := (others => '0');
 
   return addr;

  end get_wrap_addr;

-- pragma translate_off
  function check_wrap_parameters(addr, size, len : std_ulogic_vector) return std_ulogic is

    variable tmp : std_ulogic := '0';
  
  begin 
    
    if (to_integer(unsigned(len)) + 1 /= 2) and 
       (to_integer(unsigned(len)) + 1 /= 4) and
       (to_integer(unsigned(len)) + 1 /= 8) and 
       (to_integer(unsigned(len)) + 1 /= 16) then
      tmp := '1';
      assert false
        report "axi432 to VCI bridge: Invalid LEN in WRAP mode"
      severity failure;
    end if;
   
  
    if unsigned(size) /= 0 then 
      if unsigned(addr(to_integer(unsigned(size)) - 1 downto 0)) /= 0 then  
      tmp := '1';
        assert false
          report "axi432 to VCI bridge: Wrap start address not aligned on size"
        severity failure;
      end if;
    end if;

    return tmp;

  end function check_wrap_parameters;
-- pragma translate_on

  function axi432_m2s_address_channel_type2axi432_m2s_read_address_channel_type(v: axi432_m2s_address_channel_type) return axi432_m2s_read_address_channel_type is
    variable r: axi432_m2s_read_address_channel_type;
  begin
    r.arid    := v.aid;
    r.araddr  := v.aaddr;
    r.arlen   := v.alen;
    r.arsize  := v.asize;
    r.arburst := v.aburst;
    r.arlock  := v.alock;
    r.arcache := v.acache;
    r.arprot  := v.aprot;
    r.arvalid := v.avalid;
    return r;
  end function axi432_m2s_address_channel_type2axi432_m2s_read_address_channel_type;

  function axi432_m2s_address_channel_type2axi432_m2s_write_address_channel_type(v: axi432_m2s_address_channel_type) return axi432_m2s_write_address_channel_type is
    variable r: axi432_m2s_write_address_channel_type;
  begin
    r.awid    := v.aid;
    r.awaddr  := v.aaddr;
    r.awlen   := v.alen;
    r.awsize  := v.asize;
    r.awburst := v.aburst;
    r.awlock  := v.alock;
    r.awcache := v.acache;
    r.awprot  := v.aprot;
    r.awvalid := v.avalid;
    return r;
  end function axi432_m2s_address_channel_type2axi432_m2s_write_address_channel_type;

  function axi432_m2s_read_address_channel_type2axi432_m2s_address_channel_type(v: axi432_m2s_read_address_channel_type) return axi432_m2s_address_channel_type is
    variable r: axi432_m2s_address_channel_type;
  begin
    r.aid    := v.arid;
    r.aaddr  := v.araddr;
    r.alen   := v.arlen;
    r.asize  := v.arsize;
    r.aburst := v.arburst;
    r.alock  := v.arlock;
    r.acache := v.arcache;
    r.aprot  := v.arprot;
    r.avalid := v.arvalid;
    return r;
  end function axi432_m2s_read_address_channel_type2axi432_m2s_address_channel_type;

  function axi432_m2s_write_address_channel_type2axi432_m2s_address_channel_type(v: axi432_m2s_write_address_channel_type) return axi432_m2s_address_channel_type is
    variable r: axi432_m2s_address_channel_type;
  begin
    r.aid    := v.awid;
    r.aaddr  := v.awaddr;
    r.alen   := v.awlen;
    r.asize  := v.awsize;
    r.aburst := v.awburst;
    r.alock  := v.awlock;
    r.acache := v.awcache;
    r.aprot  := v.awprot;
    r.avalid := v.awvalid;
    return r;
  end function axi432_m2s_write_address_channel_type2axi432_m2s_address_channel_type;


end package body axi432_pkg;
