--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) 2009-2015 - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

use std.textio.all;

library ieee;
use ieee.std_logic_1164.all;

use work.axi4lite32_pkg.all;
use work.axi4lite32_master_pkg.all;

entity axi4lite32_master is
  port(
    clk:         in  std_ulogic;
    rstn:       in  std_ulogic;
    axi_m2s:     out axi4lite32_m2s_type;
    axi_s2m:     in  axi4lite32_s2m_type
  );

end entity axi4lite32_master;

architecture arc of axi4lite32_master is

  signal axi4lite32_local:   axi4lite32_m2s_type; --* Local host initiator to target interface

  signal srstn_local:      std_ulogic; --* Local reset
  constant cmdfile: string :="axilite_sim.dat";
begin

  axi_m2s  <= axi4lite32_local;

  process

    type state_type is (parsing, resetting, waiting_write, waiting_read, ending);

    variable state:        state_type;
    variable are:          axi4lite32_read_master_emulator;
    variable awe:          axi4lite32_write_master_emulator;
    variable cnt:          integer;
    variable good:         boolean;
    variable l:            line;
    variable ln:           natural;
    variable cn:           natural;
    variable eof:          boolean;
    variable cmd:          cmd_type;
    variable val:          std_ulogic;
    variable err:          std_ulogic;

    file cf:               text;

  begin

    -- Initializations
    axi4lite32_local <= axi4lite32_m2s_none;
    free(are);
    free(awe);
    state    := parsing;
    cnt      := 1000;
    good     := true;
    file_open(cf, cmdfile, read_mode);
    l   := null;
    ln  := 1;
    cn  := 1;
    eof := false;
    cmd := RS;

    
    wait until rising_edge(clk) and rstn = '1';

    -- Main loop
    loop

      -- axi4lite32 read address channel requests
      if axi4lite32_local.arvalid = '1' and axi_s2m.arready = '1' then -- If request acknowledged
        axi4lite32_local.arvalid <= '0'; -- Stop requesting
      end if;
      if (axi4lite32_local.arvalid = '0' or axi_s2m.arready = '1') and are.raccnt /= 0 then
        play(are, axi4lite32_local); -- Submit next request and pop it out of requests FIFO
      end if;

      -- axi4lite32 write address channel requests
      if axi4lite32_local.awvalid = '1' and axi_s2m.awready = '1' then -- If request acknowledged
        axi4lite32_local.awvalid <= '0'; -- Stop requesting
      end if;
      if (axi4lite32_local.awvalid = '0' or axi_s2m.awready = '1') and awe.waccnt /= 0 then
     --   play(awe, axi4lite32_local); -- Submit next request and pop it out of requests FIFO
      end if;

      -- axi4lite32 write data channel requests
      if axi4lite32_local.wvalid = '1' and axi_s2m.wready = '1' then -- If request acknowledged
        axi4lite32_local.wvalid <= '0'; -- Stop requesting
      end if;
      if (axi4lite32_local.wvalid = '0' or axi_s2m.wready = '1') and awe.wdccnt /= 0 then
        play(awe, axi4lite32_local); -- Submit next request and pop it out of requests FIFO
      end if;

      -- axi4lite32 read data acknoledge

    
        axi4lite32_local.rready <= '1'; -- Randomly acknowledge responses


      if axi_s2m.rvalid = '1' and axi4lite32_local.rready = '1' then -- If response acknowledged
        check(are, axi_s2m); -- Check reponse pop it out of responses FIFO
      end if;


      -- axi4lite32 write response acknoledge
 
        axi4lite32_local.bready <= '1'; -- Randomly acknowledge responses
 

      if axi_s2m.bvalid = '1' and axi4lite32_local.bready = '1' then -- If response acknowledged
        check(awe, axi_s2m); -- Check reponse pop it out of responses FIFO
      end if;

      -- Reset
      if rstn = '0' then
        free(are);
        free(awe);
      end if;

      -- State machine
      case state is
        when resetting =>
          cnt := cnt - 1;
          if cnt = 0 then
            state := parsing;
          end if;

        when ending =>
          is_empty(are, good);
          if good then
            is_empty(awe, good);
            exit when good;
          end if;

        when waiting_write =>
          cnt := cnt - 1;
          is_empty(awe, good);
          if good then
            state := parsing;
          elsif cnt = 0 then
            assert false report "Timeout exceeded while waiting for end of target WR axi4lite32 transactions" severity failure;
          end if;
        
        when waiting_read =>
          cnt := cnt - 1;
          is_empty(are, good);
          if good then
            state := parsing;
          elsif cnt = 0 then
            assert false report "Timeout exceeded while waiting for end of target RD axi4lite32 transactions" severity failure;
          end if;

        when parsing => -- Parsing of command file
          while state = parsing loop
            get_cmd_token(cf, l, ln, cn, cmd, eof); -- Read next command
            if eof then -- End of command file
              state := ending;
              exit;
            end if;
            case cmd is
              when RS =>
                get_dec_token(cf, l, ln, cn, cnt);
                state := resetting;
    
                exit;
              when ER =>
              -- Wait end of read
                get_dec_token(cf, l, ln, cn, cnt);
                state := waiting_read;
                exit;
              when EW =>
              -- Wait end of write
                get_dec_token(cf, l, ln, cn, cnt);
                state := waiting_write;
                exit;
              when WR =>
                read_vw(cf, l, ln, cn, awe);
              when RD =>
                read_vr(cf, l, ln, cn, are);
              when QT =>
                state := ending;
                exit;
            end case;
          end loop;
      end case;
      wait until rising_edge(clk);
    end loop;
    wait;

  end process;

end architecture arc;
