--
-- SecBus ( http://secbus.telecom-paristech.fr/ ) - This file is part of SecBus
-- Copyright (C) - Telecom ParisTech
-- Contacts: contact-secbus@telecom-paristech.fr
--
-- This file must be used under the terms of the CeCILL.
-- This source file is licensed as described in the file COPYING, which
-- you should have received as part of this distribution. The terms
-- are also available at
-- http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
--

library ieee;
use ieee.std_logic_1164.all;

entity fifo is
  generic (depth: positive := 32);
  port ( 
    clk	  : in  std_ulogic;
    rstn	: in  std_ulogic;
    ce   	: in  std_ulogic;
    init 	: in  std_ulogic;
    write	: in  std_ulogic;
    wdata	: in  std_ulogic_vector (31 downto 0);
    read	: in  std_ulogic;
    rdata	: out std_ulogic_vector (31 downto 0);
    empty	: out std_ulogic;
    full	: out std_ulogic
  );
end entity fifo;

architecture rtl of fifo is

  type fifo_memory is array (0 to depth - 1) of std_ulogic_vector(31 downto 0);
  signal memory : fifo_memory;
  signal wrptr  : natural range 0 to depth - 1;
  signal rdptr  : natural range 0 to depth - 1;

begin

  -- Memory Pointer Process
  fifo_proc : process (clk)
    variable looped : boolean;
  begin
    if rising_edge(clk) then
      if rstn = '0' then
        wrptr <= 0;
        rdptr <= 0;
        looped := false;
        full  <= '0';
        empty <= '1';
        rdata <= (others => '0');
      elsif ce = '1' then
        rdata <= memory(rdptr);
        if init = '1' then
          wrptr <= 0;
          rdptr <= 0;
          looped := false;
          full  <= '0';
          empty <= '1';
        end if;
        if (read = '1') then
          if ((looped = true) or (wrptr /= rdptr)) then
            -- Update RdPtr pointer as needed
            if (rdptr = depth - 1) then
              rdptr <= 0;
              looped := false;
            else
              rdptr <= rdptr + 1;
            end if;
          end if;
        end if;
        if (write = '1') then
          if ((looped = false) or (wrptr /= rdptr)) then
            -- Write Data to Memory
            memory(wrptr) <= wdata;
            -- Increment WrPtr pointer as needed
            if (wrptr = depth - 1) then
              wrptr <= 0;
              looped := true;
            else
              wrptr <= wrptr + 1;
            end if;
          end if;
        end if;
        -- Update empty and full flags
        if (wrptr = rdptr) then
          if looped then
            full <= '1';
          else
            empty <= '1';
          end if;
        else
          empty	<= '0';
          full	<= '0';
        end if;
      end if; -- rstn
    end if; -- rising_edge(clk)
  end process fifo_proc;

end architecture rtl;
